<?php

class GGPackage {

	private $attachment_id;
	private $attachment_url;
	private $attachment_file;
	private $remote_url;
	private $local_url;
	private $is_pano = false;
	private $is_object = false;
	private $player_file = "";
	private $skin_file = "";
	private $translations_file = "";
	private $xml_file = "";
	private $preview_file = "";
	public $use_preview = false;
	public $preview_only = false;
	public $use_async = false;
	public $show_play_button = true;
	public $start_node = "";
	public $start_view = "";
	public $width = "500px";
	public $height = "500px";
	public $pano_player_version = "package";
	public $object_player_version = "package";
	private $json;
	private static $code_uid = 0;
	private $viewer;

	public function __construct( $viewer = null, $attachment = null ) {
		$this->viewer = $viewer;
		if(isset($attachment["start_preview"]) && $attachment["start_preview"] == "true"){
           $this->use_preview           = true;
		}else{
			$this->use_preview           = false;
		}
		
		$this->pano_player_version   = "package";
		$this->object_player_version = "package";
		
		if ( isset( $attachment ) ) {
			$this->from_attachment( $attachment );
		}
	}

	public function from_attachment( $attachment ) {
		$this->attachment_id   = $attachment["id"];
		$attachment_url        =  $attachment["package_url"];
		$this->attachment_file = $attachment["package_path"];
		$this->plugin_path = $attachment["plugin_path"];
		$this->plugin_url = $attachment["plugin_path"];
		$filename              = strrchr( $attachment_url, '/' );
		$filename              = str_replace( '-', '_', $filename );
		$attachment_url        = substr( $attachment_url, 0, strrpos( $attachment_url, '/' ) ) . $filename;
		$this->local_url       = $attachment_url;
		// remove domain?
		// $this->local_url       = $this->url_from_local($this->local_url);
		$this->attachment_url = $attachment_url;
		if ( substr( $this->local_url, - 1 ) != "/" ) {
			$this->local_url .= "/";
		}
		if ( $this->file_in_package( "gginfo.json" ) ) {
			if ( ( $json_content = file_get_contents( $this->abs_folder() . "/gginfo.json" ) ) != false ) {
				$json_content = utf8_encode( $json_content );
				$this->parse_gginfo_json( $json_content );
			}
		} else {
			if ( file_exists( $this->abs_folder()."/pano2vr_player.js" ) ) {
				$this->is_pano     = true;
				$this->player_file = "pano2vr_player.js";
				$this->xml_file    = "pano.xml";
			}
			if ( file_exists( $this->abs_folder()."/object2vr_player.js" ) ) {
				$this->is_object   = true;
				$this->player_file = "object2vr_player.js";
				$this->xml_file    = "object.xml";
			}
			if ( file_exists( $this->abs_folder()."/skin.js" ) ) {
				$this->skin_file = "skin.js";
			}
			if ( file_exists( $this->abs_folder()."/translations.js" ) ) {
				$this->translations_file = "translations.js";
			}
			if ( file_exists( $this->abs_folder()."/preview.jpg" ) ) {
				$this->preview_file = "preview.jpg";
			} else {
				$this->preview_file = "empty.jpg";
			}
		}
	}

	public function set_from_url( $url ) {
		$this->attachment_id   = "r";
		$this->remote_url      = $url;
		$this->attachment_url  = "";
		$this->attachment_file = "";
		$remote_json_file      = $url . "gginfo.json";
		$transient             = 'ggexurl_' . md5( $url );
		$json_response         = get_transient( $transient );

		if ( false === $json_response ) {
			// Transient expired, refresh the data
			$json_response = wp_remote_get( $remote_json_file, [ 'sslverify' => false ] );
			$code          = wp_remote_retrieve_response_code( $json_response );
			set_transient( $transient, $json_response, ( $code == "200" ) ? 3600 : 60 );
		} else {
			$code = wp_remote_retrieve_response_code( $json_response );
		}
		$json_content = wp_remote_retrieve_body( $json_response );
		if ( $code == "200" ) {
			$this->parse_gginfo_json( $json_content );
		}
	}

	public function file_in_package( $file ) {
		$extract_path_local = $this->abs_folder();
		if ( file_exists( $extract_path_local . "/" . $file ) ) {
			return true;
		} else {
			return false;
		}
	}

	public function url_from_local( $url ) {
		$url_from_local = $url;
		$url_parts      = parse_url( $url );
		// 	if ($url_parts['host'] == $_SERVER['SERVER_NAME'])
		// 	{
		$url_from_local = $url_parts['path'];

		// 	}
		return $url_from_local;
	}

	public function folder() {
		$path_parts = pathinfo( $this->attachment_file );
		$filename   = str_replace( '-', '_', $path_parts['filename'] );
		$filename   = str_replace( ' ', '_', $filename );

		return $filename;
	}

	public function unique_player() {
		if ( $this->remote_url ) {
			return "";
		} else {
			$filename = $this->folder();
			if ( is_numeric( substr( $filename, 0, 1 ) ) ) {
				$filename = '_' . $filename;
			}

			return $filename;
		}
	}

	public function abs_folder() {
		//return  $this->plugin_url; 
		return $path_parts = $this->attachment_file; 
		//return $path_parts['dirname'] . "/" . $this->folder();
	}

	public function file_url( $file ) {
		$attachmentURL = $this->attachment_url;
		$attachmentURL = substr( $attachmentURL, 0, strrpos( $attachmentURL, '/' ) ) . $this->filename;
		$extract_path  = substr( $attachmentURL, 0, strrpos( $attachmentURL, '.' ) );

		return $extract_path . "/" . $file;
	}

	public function to_url( $file ) {

		if ( $this->remote_url ) {
			return $this->remote_url . $file;
		} else {
			return $this->local_url . $file;
		}
	}



	public function parse_gginfo_json( $json_content ) {
		$json = json_decode( $json_content );
		
		if ( ! $json ) {
			return;
		}
		$this->json = $json;
		if ( isset( $json->{'type'} ) ) {
			$json_type  = $json->{'type'};
			// only newer player version have the async function, but they where introduced at the same time as as the json file in the package
			$this->use_async = true;
			$this->is_pano   = ( $json_type == 'panorama' );
			$this->is_object = ( $json_type == 'object' );
			if ( $this->is_pano ) {
				$this->xml_file = "pano.xml";
			}
			if ( $this->is_object ) {
				$this->xml_file = "object.xml";
			}
		}
		if ( isset( $json->{'player'} ) ) {
			$json_player = $json->{'player'};
			$json_js_player = $json_player->{'js'};
			if ( isset( $json_js_player ) ) {
				$this->player_file = $json_js_player;
			}
		}
		if ( isset( $json->{'skin'} ) ) {
			$json_skin = $json->{'skin'};
			$json_skin_file = $json_skin->{'js'};
			if ( isset( $json_skin_file ) ) {
				$this->skin_file = $json_skin_file;
			}
		}
		if ( isset($json->{'translations'}) ) {
			$json_translations = $json->{'translations'};
			$json_translations_file = $json_translations->{'js'};
			if ( isset( $json_translations_file ) ) {
				$this->translations_file = $json_translations_file;
			}
		}
		if ( isset( $json->{'preview'} ) ) {
			$json_preview = $json->{'preview'};
			$json_preview_file = $json_preview->{'img'};
			if ( isset( $json_preview_file ) ) {
				$this->preview_file = $json_preview_file;
			} else {
//				$attributes['start_preview'] = 'false';
			}
		} else {
//			$attributes['start_preview'] = 'false';
		}

	}

	public function get_html_code( $attributes ) {


		$ID = "_" . GGPackage::$code_uid . "_" . $attributes["id"];
		GGPackage::$code_uid ++;

		$document = JFactory::getDocument();
        




		if ( $this->json ) {
			$json_externals = $this->json->{'external'};
			if ( isset( $json_externals ) ) {
				$js_files = $json_externals->{'js'};
				if ( isset( $js_files ) ) {
					$index = 0;
					foreach ( $js_files as $js_file ) {
						if ( substr( $js_file, 0, 4 ) === "http" ) {
							$document->addScript($js_file);
						} else {
							if ( ( substr( $js_file, 0, 6 ) == 'webvr/' ) ||
								( substr( $js_file, 0, 6 ) == 'webxr/' ) ) { // only load the webvr scripts from one source
								//echo $this->to_url( $js_file); die;
								$document->addScript($this->to_url( $js_file));
								
							} else {
								$document->addScript($this->to_url( $js_file));
							}
						}
						$index ++;
					}
				}
				$css_files = $json_externals->{'css'};
				if ( isset( $css_files ) ) {
					$index = 0;
					foreach ( $css_files as $css_file ) {
						if ( substr( $css_file, 0, 4 ) === "http" ) {
							$document->addStyleSheet($this->to_url( $css_file));
						} else {
							$document->addStyleSheet($this->to_url( $css_file));
						}
						$index ++;
					}
				}
			}
		}
		$width = "500px";
		$height = "500px";
        if($attributes["width"] != ""){
          $width  = $attributes["width"];
        }
        if($attributes["height"] != ""){
        	$height = $attributes["height"];
        }
		if ( is_numeric( $width ) ) {
			$width = $width . "px";
		}
		if ( is_numeric( $height ) ) {
			$height = $height . "px";
		}
		$html = "<div id='ggpkg_container" . $ID . "' style='width:" . $width . "; height:" . $height . "; position: relative;'>\n";
		if ( $this->use_preview ) {

			$html .= "<div style='width:100%; height:100%; overflow: hidden; position:relative;'>\n";
			if ( $this->preview_file ) {
				$html .= "<img src='" . $this->to_url( $this->preview_file ) . "' alt='' onclick='startPlayer" . $ID . "();' style='width:100%;min-width:100%; max-width: 10000px; min-height: 100%; max-height: 10000px;'>\n";
			}
			if ( $this->show_play_button ) {
				$html .= "<img src='" . juri::root() . "plugins/content/ggpkg_import/include/play.png" . "' alt='' onclick='startPlayer" . $ID . "();' style='width: 180px; height: 180px; display: block;";
			}
			if ( substr( $width, - 1 ) == '%' ) {
				if ( substr( $height, - 1 ) == '%' ) {
					$html .= "top: 50%; margin-top: -90px; left: 50%; margin-left: -90px;";
				} else {
					$html .= "top: " . strval( $height / 2 - 90 ) . "px; left: 50%; margin-left: -90px;";
				}
			} else {
				if ( substr( $height, - 1 ) == '%' ) {
					$html .= "top: 50%; margin-top: -90px; left: " . strval( $width / 2 - 90 ) . "px;";
				} else {
					$html .= "top: " . strval( $height / 2 - 90 ) . "px; left: " . strval( $width / 2 - 90 ) . "px;";
				}
			}
			$html .= "box-shadow: none; position: absolute;'>\n";
			$html .= "</div>\n";
		} else {
			$html .= "Loading...\n";
		}
		$html .= "</div>\n";


		if ( ( $this->remote_url == "" ) && ( $this->is_pano ) && ( $this->pano_player_version != "package" ) // Remote packages should load the remote player to avoid CORS trouble.
			&& ( file_exists( $attributes["plugin_path"] . "/pano2vr_player/" . $this->pano_player_version . "/pano2vr_player.js" ) ) ) {
			
			$document->addScript($attributes["plugin_url"]."/pano2vr_player/" . $this->pano_player_version . "/pano2vr_player.js");

		} elseif ( ( $this->remote_url == "" ) && ( $this->is_object ) && ( $this->object_player_version != "package" )
			&& ( file_exists( $attributes["plugin_path"] . "/object2vr_player/" . $this->object_player_version . "/object2vr_player.js" ) ) ) {
			$document->addScript($attributes["plugin_url"]. "/object2vr_player/" . $this->object_player_version . "/object2vr_player.js");
		} else {
			$html .= "<script type='text/javascript' src='" . $this->to_url( $this->player_file ) . "'></script>\n";
		}

		if ( $this->translations_file ) {
			$html .= "<script type='text/javascript' src='" . $this->to_url( $this->translations_file ) . "'></script>\n";
		}
		if ( $this->skin_file ) {
			$html .= "<script type='text/javascript' src='" . $this->to_url( $this->skin_file ) . "'></script>\n";
		}
		$html .= "<script type='text/javascript'> \n";
        $uplayer = $this->unique_player();
		if ( $this->is_object ) {
			$jsObj         = "obj" . $ID;
			$jsClassPrefix = "object2vr";
		} else {
			$jsObj         = "pano" . $ID;
			$jsClassPrefix = "pano2vr";
		}
		if ( $uplayer != "" ) {
			$play_js = "if  (typeof " . $uplayer . "_" . $jsClassPrefix . "Player != 'undefined') {\n";
			$play_js .= "\t" . $jsObj . "=new " . $uplayer . "_" . $jsClassPrefix . "Player('ggpkg_container" . $ID . "');\n";
			$play_js .= "} else {\n";
			$play_js .= "\t" . $jsObj . "=new " . $jsClassPrefix . "Player('ggpkg_container" . $ID . "');\n";
			$play_js .= "}\n";
		} else {
			$play_js = $jsObj . "=new " . $jsClassPrefix . "Player('ggpkg_container" . $ID . "');\n";
		}
		if ( $this->start_node != "" ) {
			$play_js .= $jsObj . ".startNode = '" . $this->start_node . "';\n";
		}
		if ( $this->start_view != "" ) {
			$view_params = explode( "/", $this->start_view, 4 );
			if ( sizeof( $view_params ) >= 3 ) {
				$play_js .= $jsObj . ".startView = {\n";
				$play_js .= "   pan :" . floatval( trim( $view_params[0] ) ) . ",\n";
				$play_js .= "   tilt :" . floatval( trim( $view_params[1] ) ) . ",\n";
				$play_js .= "   fov :" . floatval( trim( $view_params[2] ) ) . ",\n";
				if ( sizeof( $view_params ) > 3 ) {
					$play_js .= "   projection :" . intval( trim( $view_params[3] ) ) . ",\n";
				}
				$play_js .= "};\n";
			}
		}
		if ( $this->skin_file ) {
			if ( $uplayer != "" ) {
				$play_js .= "if  (typeof " . $uplayer . "_" . $jsClassPrefix . "Skin != 'undefined') {\n";
				$play_js .= "\tskin" . $ID . "=new " . $uplayer . "_" . $jsClassPrefix . "Skin(" . $jsObj . ", '" . $this->to_url( "" ) . "');\n";
				$play_js .= "} else {\n";
				$play_js .= "\tskin" . $ID . "=new " . $jsClassPrefix . "Skin(" . $jsObj . ", '" . $this->to_url( "" ) . "');\n";
				$play_js .= "}\n";
			} else {
				$play_js .= "skin" . $ID . "=new " . $jsClassPrefix . "Skin(" . $jsObj . ", '" . $this->to_url( "" ) . "');\n";
			}
		}
		if ( $this->use_async ) {
			$play_js .= $jsObj . ".readConfigUrlAsync('" . $this->to_url( $this->xml_file ) . "');\n";
		} else {
			$play_js .= $jsObj . ".readConfigUrl('" . $this->to_url( $this->xml_file ) . "');\n";
		}
		$play_js .= "var contentOverflow = document.getElementById('content') ? document.getElementById('content').style.overflow : '';\n";
		
		// make the code look nice
//		$play_js="\t".str_replace("\n","\n\t",$play_js);
		if ( $this->use_preview ) {
			if ( ! $this->preview_only ) {
				$html .= "function startPlayer" . $ID . "() {\n";
				$html .= $play_js;
				$html .= "}\n";
			}
		} else {
			$html .= "window.addEventListener('load',function() {\n" . $play_js . "\n});\n";
		}
		$html .= "</script>\n";

		return $html;
	}
}